/**
 *  \file alakzat.h
 *  Alakzat alaposztály deklarációja
 *
 * *** Extra teszt az explicit destruktorhívás megcsípéséhez. ***
 * *** Tilos a destruktort explicit meghívni, főleg saját tagfüggvényből! ***
 *
 */
#ifndef ALAKZAT_H
#define ALAKZAT_H

#include <iostream>         // ostream miatt kell
#include "memtrace.h"       // nagy valószínűséggel minden használt headert behúz
#include "pont.h"
#include "szin.h"

/// Alakzat abszrakt osztály
class Alakzat {
     Pont p0;                   /// Alakzat origója, vagy bázis pontja.
     /// Minden további adatot ehhez relatívan tárolunk. Geometriai értelemben nem feltétlenül origó.
     Szin sz;                   /// alakzat színe
     char *dummy;               /// Explicit destrutorhívás teszteléséhez
     static const char* juj;    /// Nagyon fájdalmas üzenet
public:
    /// Konstruktor
    /// @param p0 - kezdőpont
    /// @param sz - szín és átlátszóság
    Alakzat(const Pont& p0, const Szin& sz) :p0(p0), sz(sz), dummy(new char[100]) {}

	/// Másoló konstruktor,
	/// mert van dinamikus adattag, amit kezel
    Alakzat(const Alakzat& rhs) {
        dummy = NULL;
        *this = rhs;
    }

	/// Értékadó operator,
	/// mert van dinamikus adattag, amit kezel
    Alakzat& operator=(const Alakzat& rhs) {
        if (this != &rhs) {
#ifdef MEMTRACE
		/// Explicit destrutorhívást próbálja felderíteni.
		/// Elhangzott előadáson, hogy tilos, de mégis van aki megpróbálja.
            if (!memtrace::poi_check(dummy))
                std::cerr << juj;
#endif
            delete[] dummy;
            dummy = new char [100];
            p0 = rhs.p0;
            sz = rhs.sz;
        }
        return *this;
    }


    /// Pozíció lekérdezése
    /// @return alakzat origója
    const Pont& getp0() const { return p0; }

    /// Szin lekérdezése
    /// @return alakzat színe
    const Szin& getsz() const { return sz; }

    /// mozgat: origóját eltolja
    /// @param d - eltolás vektora
    void mozgat(const Pont& d);

    /// rajzol
    virtual void rajzol() const = 0;

    /// Destruktor virtuális, hogy lehessen dinamikus adattagja
    /// a származtatottnak
    virtual ~Alakzat() {
#ifdef MEMTRACE
		/// Explicit destrutorhívást próbálja felderíteni.
		/// Elhangzott előadáson, hogy tilos, de mégis van aki megpróbálja.
        if (!memtrace::poi_check(dummy))
            std::cerr << juj;
#endif
        delete[] dummy;
    }
};

/// Globális << operátor az Alakzat adatainak kiírásához
/// @param os - ostream
/// @param a - alakzat
/// @return os - ostream
std::ostream& operator<<(std::ostream& os, const Alakzat& a);

#endif // ALAKZAT_H
